import { EventEmitter } from 'events';

/**
 * Configuration options for database initialization.
 */
interface DatabaseOptions {
    path?: string;
    inMemory?: boolean;
    createIfMissing?: boolean;
    encrypt?: boolean;
    encryptionMethod?: 'aes-256-cbc' | 'aes-256-gcm' | 'chacha20-poly1305';
    encryptionKey?: string;
    encryptionKDF?: 'pbkdf2' | 'scrypt' | 'argon2';
    autoFlush?: boolean;
    flushInterval?: number;
    cacheDocuments?: boolean;
    maxCacheSize?: number;
    enableIndexing?: boolean;
    schemaValidation?: 'strict' | 'warn' | 'ignore';
    schemaPath?: string;
    debug?: boolean;
    logLevel?: 'error' | 'warn' | 'info' | 'debug';
}
/**
 * Definition of a single field in a document schema.
 */
interface SchemaField {
    type: 'string' | 'number' | 'boolean' | 'object' | 'array' | 'date' | 'buffer';
    required?: boolean;
    default?: any;
    unique?: boolean;
    index?: boolean;
    min?: number;
    max?: number;
    pattern?: RegExp;
    enum?: any[];
    ref?: string;
    validate?: (value: any) => boolean | string;
}
/**
 * Complete schema definition for a collection.
 */
interface Schema {
    [fieldName: string]: SchemaField;
}
/**
 * Base document interface with system metadata.
 */
interface Document {
    _id: string;
    _createdAt: Date;
    _updatedAt: Date;
    _version?: number;
    [key: string]: any;
}
/**
 * Lightweight document metadata without full content.
 */
interface DocumentMetadata {
    id: string;
    createdAt: Date;
    updatedAt: Date;
    size: number;
    checksum?: string;
    version?: number;
}
/**
 * Supported query operators for filtering documents.
 */
type QueryOperator = '$eq' | '$ne' | '$gt' | '$gte' | '$lt' | '$lte' | '$in' | '$nin' | '$exists' | '$regex' | '$text' | '$and' | '$or' | '$nor' | '$not';
/**
 * Query condition with operator and value.
 */
interface QueryCondition {
    [operator: string]: any;
}
/**
 * MongoDB-like query filter for finding documents.
 */
interface QueryFilter {
    [field: string]: any | QueryCondition;
}
/**
 * Options for query operations like sorting, pagination, and projection.
 */
interface QueryOptions {
    limit?: number;
    skip?: number;
    sort?: {
        [field: string]: 1 | -1;
    } | Array<[string, 1 | -1]>;
    projection?: {
        [field: string]: 0 | 1;
    };
    explain?: boolean;
    hint?: string;
    timeout?: number;
}
/**
 * Configuration options for collections.
 */
interface CollectionOptions {
    schema?: Schema;
    encrypt?: boolean;
    compression?: boolean;
    indexes?: IndexDefinition[];
    maxDocuments?: number;
    autoIndex?: boolean;
    maxCacheSize?: number;
}
/**
 * Definition of a database index.
 */
interface IndexDefinition {
    fields: {
        [field: string]: 1 | -1;
    };
    unique?: boolean;
    sparse?: boolean;
    name?: string;
}
/**
 * Result of a single document insert operation.
 */
interface InsertResult {
    id: string;
    success: boolean;
    document: Document;
    insertedCount: number;
}
/**
 * Result of a bulk document insert operation.
 */
interface InsertManyResult {
    insertedIds: string[];
    insertedCount: number;
    success: boolean;
}
/**
 * Result of a document update operation.
 */
interface UpdateResult {
    modifiedCount: number;
    success: boolean;
    upsertedId?: string;
    upsertedCount?: number;
}
/**
 * Result of a document delete operation.
 */
interface DeleteResult {
    deletedCount: number;
    success: boolean;
}
/**
 * Result of a document find operation with pagination metadata.
 */
interface FindResult<T = Document> {
    documents: T[];
    total: number;
    hasMore: boolean;
    page?: number;
    totalPages?: number;
}
/**
 * Fluent interface for building complex queries.
 */
interface QueryBuilder$1<T = Document> {
    where(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    and(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    or(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    sort(field: string, direction: 1 | -1): QueryBuilder$1<T>;
    limit(count: number): QueryBuilder$1<T>;
    skip(count: number): QueryBuilder$1<T>;
    select(fields: string[]): QueryBuilder$1<T>;
    execute(): Promise<FindResult<T>>;
    findOne(): Promise<T | null>;
    count(): Promise<number>;
}
/**
 * Database transaction interface (placeholder for future implementation).
 */
interface Transaction {
    commit(): Promise<void>;
    rollback(): Promise<void>;
    collection<T = Document>(name: string): any;
}
/**
 * Database-wide statistics and metrics.
 */
interface DatabaseStats {
    collections: number;
    totalDocuments: number;
    totalSize: number;
    indexes: number;
    uptime: number;
}
/**
 * Event types emitted by the database.
 */
interface DatabaseEvents {
    'document:inserted': (collection: string, document: Document) => void;
    'document:updated': (collection: string, document: Document) => void;
    'document:deleted': (collection: string, documentId: string) => void;
    'collection:created': (name: string) => void;
    'collection:dropped': (name: string) => void;
    error: (error: Error) => void;
}

/**
 * Thin wrapper around the Node.js `fs` module that provides collection-aware
 * persistence utilities. Documents are stored as individual JSON files inside
 * a directory named after the collection.
 */
declare class FileStorage {
    private basePath;
    private ensuredDirs;
    /**
     * @param basePath - Root directory where all collections will be stored.
     */
    constructor(basePath: string);
    /** Ensure a directory exists (memoised for performance). */
    ensureDirectory(path: string): Promise<void>;
    /**
     * Persist a single document.
     *
     * @param collectionPath - Collection folder relative to `basePath`.
     * @param document       - Document with metadata to write.
     */
    writeDocument(collectionPath: string, document: Document): Promise<void>;
    /**
     * Read a single document. Returns `null` if the file doesn't exist.
     */
    readDocument(collectionPath: string, documentId: string): Promise<Document | null>;
    /** Read every document stored in a collection directory. */
    readAllDocuments(collectionPath: string): Promise<Document[]>;
    /** Delete a single document file. Returns `false` if it didn't exist. */
    deleteDocument(collectionPath: string, documentId: string): Promise<boolean>;
    /** Check existence of a document without reading it. */
    documentExists(collectionPath: string, documentId: string): Promise<boolean>;
    /** Lightweight metadata fetch that avoids parsing the whole file. */
    getDocumentMetadata(collectionPath: string, documentId: string): Promise<DocumentMetadata | null>;
}

/**
 * Public facade that combines CRUD (DocumentOperations) and query (QueryOperations)
 * capabilities for a collection.
 *
 * @typeParam T - Shape of the documents stored in this collection.
 */
declare class Collection<T = Document> {
    private documentOps;
    private queryOps;
    /**
     * Create a new Collection instance.
     *
     * @param name     - Logical name of the collection (also directory name on disk).
     * @param storage  - Low-level storage engine implementation.
     * @param options  - Optional collection-specific configuration.
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /** Insert a single document. */
    insert(data: Partial<T>): Promise<InsertResult>;
    /** Insert many documents in bulk. */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /** Update all documents that match a filter. */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /** Update existing documents or insert a new one if nothing matches. */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /** Delete all matching documents. */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /** Delete the first document that matches the filter. */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /** Manually create an index. */
    createIndex(definition: IndexDefinition): Promise<void>;
    /**
     * Find documents matching a filter.
     *
     * @param filter   - Query criteria (MongoDB-like).
     * @param options  - Pagination, sorting, projection …
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /** Return first document that matches. */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /** Lookup a document by its `_id`. */
    findById(id: string): Promise<T | null>;
    /** Start a fluent `QueryBuilder` chain. */
    query(): QueryBuilder$1<T>;
    /** Count matching documents. */
    count(filter?: QueryFilter): Promise<number>;
    /** Check if the collection currently has zero documents. */
    isEmpty(): Promise<boolean>;
    /** Clear in-memory LRU/cache for this collection. */
    clearCache(): void;
    /** Return size, index & cache statistics. */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
    /** Lazily called by BaseCollection; can also be invoked manually. */
    initialize(): Promise<void>;
}

/**
 * Main database instance that manages collections and provides the public API.
 * Implements the singleton pattern for global database access.
 */
declare class NuboDB extends EventEmitter {
    private static instance;
    private options;
    private storage;
    private encryptionManager?;
    private collections;
    private isOpen;
    private startTime;
    private logger;
    /**
     * Private constructor for singleton pattern.
     *
     * @param options - Database configuration including storage path, encryption, etc.
     */
    private constructor();
    /**
     * Get the singleton database instance. Creates one if it doesn't exist.
     *
     * @param options - Required for first initialization.
     */
    static getInstance(options?: DatabaseOptions): Promise<NuboDB>;
    /**
     * Create a new database instance (bypasses singleton).
     *
     * @param options - Database configuration.
     */
    static create(options: DatabaseOptions): Promise<NuboDB>;
    /** Initialize the database and create storage directories. */
    open(): Promise<void>;
    /** Clean up resources and close all collections. */
    close(): Promise<void>;
    /**
     * Get or create a collection by name.
     *
     * @param name    - Collection name.
     * @param options - Optional collection-specific settings.
     * @returns Collection instance for CRUD operations.
     */
    collection<T = any>(name: string, options?: CollectionOptions): Collection<T>;
    /**
     * Explicitly create a new collection with optional schema.
     *
     * @param name    - Collection name.
     * @param schema  - Optional schema for validation.
     * @param options - Collection configuration.
     */
    createCollection<T = any>(name: string, schema?: Schema, options?: CollectionOptions): Promise<Collection<T>>;
    /**
     * Delete all documents in a collection and remove it from memory.
     *
     * @param name - Collection to drop.
     * @returns Success status.
     */
    dropCollection(name: string): Promise<boolean>;
    /** Get names of all currently loaded collections. */
    listCollections(): Promise<string[]>;
    /** Check if a collection is currently loaded in memory. */
    hasCollection(name: string): boolean;
    /** Get aggregated statistics across all collections. */
    getStats(): Promise<DatabaseStats>;
    /** Get a copy of current database configuration. */
    getOptions(): DatabaseOptions;
    /** Check if database is currently open and ready for operations. */
    isDatabaseOpen(): boolean;
    /** Get the storage path where database files are located. */
    getPath(): string;
    /** Clear all collection caches to free memory. */
    clearCaches(): void;
    /**
     * Create a backup of the database (placeholder implementation).
     *
     * @param backupPath - Destination for backup files.
     */
    backup(backupPath: string): Promise<void>;
    /** Perform database compaction to optimize storage (placeholder). */
    compact(): Promise<void>;
    private setupLogger;
    private log;
    on<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    once<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    off<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
}

/**
 * Fluent query builder that provides a chainable API for building complex queries.
 * Similar to MongoDB's query builder but with a more JavaScript-friendly syntax.
 *
 * @typeParam T - Document type being queried.
 */
declare class QueryBuilder<T = Document> {
    private filter;
    private options;
    private collection;
    /**
     * Create a new query builder for a collection.
     *
     * @param collection - Collection instance to query.
     */
    constructor(collection: any);
    /**
     * Add a where clause to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator ($eq, $gt, etc.).
     * @param value    - Value to compare against.
     */
    where(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Add an AND condition to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator.
     * @param value    - Value to compare against.
     */
    and(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Add an OR condition to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator.
     * @param value    - Value to compare against.
     */
    or(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Sort results by a field.
     *
     * @param field     - Field to sort by.
     * @param direction - 1 for ascending, -1 for descending.
     */
    sort(field: string, direction: 1 | -1): QueryBuilder<T>;
    /**
     * Limit the number of results returned.
     *
     * @param count - Maximum number of documents to return.
     */
    limit(count: number): QueryBuilder<T>;
    /**
     * Skip a number of results (for pagination).
     *
     * @param count - Number of documents to skip.
     */
    skip(count: number): QueryBuilder<T>;
    /**
     * Select specific fields to include in results.
     *
     * @param fields - Array of field names to include.
     */
    select(fields: string[]): QueryBuilder<T>;
    /** Execute the query and return results. */
    execute(): Promise<FindResult<T>>;
    /** Execute query and return the first result only. */
    findOne(): Promise<T | null>;
    /** Count documents matching the query. */
    count(): Promise<number>;
    /** Check if any documents match the query. */
    exists(): Promise<boolean>;
    /**
     * Find first matching document and update it.
     *
     * @param updateData - Fields to update.
     */
    findOneAndUpdate(updateData: Partial<T>): Promise<T | null>;
    /** Find first matching document and delete it. */
    findOneAndDelete(): Promise<T | null>;
}

/**
 * Handles encryption and decryption of data using AES-256 with PBKDF2 key derivation.
 * Provides both string and object encryption utilities.
 */
declare class EncryptionManager {
    private key;
    private algorithm;
    /**
     * Initialize encryption manager with a key and algorithm.
     *
     * @param encryptionKey - Password/key for encryption.
     * @param algorithm     - Encryption algorithm (default: aes-256-cbc).
     */
    constructor(encryptionKey: string, algorithm?: string);
    /**
     * Derive encryption key from password using PBKDF2.
     *
     * @param password - Password to derive key from.
     * @param salt     - Optional salt (generates random if not provided).
     */
    private deriveKey;
    /**
     * Encrypt a string using AES-256-CBC.
     *
     * @param data - String to encrypt.
     * @returns Encrypted data in format "iv:encrypted".
     */
    encrypt(data: string): string;
    /**
     * Decrypt a string that was encrypted with this manager.
     *
     * @param encryptedData - Encrypted data in "iv:encrypted" format.
     * @returns Decrypted string.
     */
    decrypt(encryptedData: string): string;
    /**
     * Encrypt a JavaScript object by serializing to JSON first.
     *
     * @param obj - Object to encrypt.
     * @returns Encrypted JSON string.
     */
    encryptObject(obj: any): string;
    /**
     * Decrypt and deserialize a JavaScript object.
     *
     * @param encryptedData - Encrypted JSON string.
     * @returns Decrypted object.
     */
    decryptObject(encryptedData: string): any;
}

/**
 * Document with system metadata automatically added by NuboDB.
 */
type DocumentWithMetadata = Document & {
    _id: string;
    _createdAt: Date;
    _updatedAt: Date;
    _version?: number;
};
/**
 * Abstract base class that provides common functionality for collections.
 * Handles initialization, caching, indexing, and metadata management.
 *
 * @typeParam T - Document type for this collection.
 */
declare abstract class BaseCollection<T = Document> {
    protected name: string;
    protected storage: FileStorage;
    protected encryptionManager?: EncryptionManager;
    protected schema?: Schema;
    protected options: CollectionOptions;
    protected cache: Map<string, T>;
    protected indexes: Map<string, Map<any, string[]>>;
    protected isInitialized: boolean;
    /**
     * Initialize the base collection with storage and options.
     *
     * @param name    - Collection name.
     * @param storage - Storage engine instance.
     * @param options - Collection configuration.
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /**
     * Load documents from storage, build indexes, and populate cache.
     * Called automatically before first operation.
     */
    initialize(): Promise<void>;
    /** Ensure collection is initialized before performing operations. */
    protected ensureInitialized(): Promise<void>;
    /** Get all documents from storage with decryption if enabled. */
    protected getAllDocuments(): Promise<T[]>;
    /**
     * Build in-memory indexes for fields marked as indexed in schema.
     *
     * @param documents - Documents to index.
     */
    protected buildIndexes(documents: T[]): Promise<void>;
    /**
     * Update indexes when documents are modified.
     *
     * @param document  - Document being modified.
     * @param operation - Type of operation (insert/update/delete).
     */
    protected updateIndexes(document: T & DocumentWithMetadata, operation: 'insert' | 'update' | 'delete'): Promise<void>;
    /**
     * Extract index key from document for compound indexes.
     *
     * @param document - Document to extract key from.
     * @param fields   - Fields to include in index key.
     */
    protected extractIndexKey(document: T & DocumentWithMetadata, fields: {
        [field: string]: 1 | -1;
    }): any;
    /** Clear the document cache to free memory. */
    clearCache(): void;
    /**
     * Get collection statistics.
     *
     * @returns Object with document count, size, index count, and cache size.
     */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
}

/**
 * Handles all document modification operations (CRUD) for a collection.
 * Extends BaseCollection to inherit caching, indexing, and initialization.
 *
 * @typeParam T - Document type for this collection.
 */
declare class DocumentOperations<T = Document> extends BaseCollection<T> {
    private queryOps;
    /**
     * Create document operations for a collection.
     *
     * @param name    - Collection name.
     * @param storage - Storage engine instance.
     * @param options - Collection configuration.
     */
    constructor(name: string, storage: any, options?: any);
    /**
     * Insert a single document with validation and metadata.
     *
     * @param data - Document data to insert.
     * @returns Insert result with generated ID and document.
     */
    insert(data: Partial<T>): Promise<InsertResult>;
    /**
     * Insert multiple documents in a single operation.
     *
     * @param documents - Array of documents to insert.
     * @returns Bulk insert result with IDs and count.
     */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /**
     * Update all documents matching a filter.
     *
     * @param filter     - Query filter to match documents.
     * @param updateData - Fields to update in matching documents.
     * @returns Update result with modified count.
     */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Update existing documents or insert new one if none match.
     *
     * @param filter     - Query filter to match documents.
     * @param updateData - Data to update or insert.
     * @returns Update result with upsert information.
     */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Delete all documents matching a filter.
     *
     * @param filter - Query filter to match documents for deletion.
     * @returns Delete result with deleted count.
     */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Delete the first document that matches a filter.
     *
     * @param filter - Query filter to match document for deletion.
     * @returns Delete result with deleted count (0 or 1).
     */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Manually create an index on specified fields.
     *
     * @param definition - Index definition with fields and options.
     */
    createIndex(definition: IndexDefinition): Promise<void>;
}

/**
 * Handles all query operations for a collection including filtering, sorting,
 * and pagination. Extends BaseCollection to inherit caching and indexing.
 *
 * @typeParam T - Document type for this collection.
 */
declare class QueryOperations<T = Document> extends BaseCollection<T> {
    /**
     * Find documents matching a filter with optional sorting and pagination.
     *
     * @param filter  - MongoDB-like query filter.
     * @param options - Query options (sort, limit, skip, projection).
     * @returns Query result with documents and metadata.
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /**
     * Find the first document matching a filter.
     *
     * @param filter - Query filter to match documents.
     * @returns First matching document or null.
     */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /**
     * Find a document by its unique ID (with cache lookup).
     *
     * @param id - Document ID to lookup.
     * @returns Document or null if not found.
     */
    findById(id: string): Promise<T | null>;
    /** Create a new QueryBuilder instance for this collection. */
    query(): QueryBuilder$1<T>;
    /**
     * Count documents matching a filter.
     *
     * @param filter - Query filter to match documents.
     * @returns Number of matching documents.
     */
    count(filter?: QueryFilter): Promise<number>;
    /** Check if collection has no documents. */
    isEmpty(): Promise<boolean>;
    /**
     * Apply filter to a list of documents.
     *
     * @param documents - Documents to filter.
     * @param filter    - Query filter to apply.
     */
    private filterDocuments;
    /**
     * Check if a document matches a query filter.
     *
     * @param document - Document to check.
     * @param filter   - Query filter to match against.
     */
    private matchesFilter;
    /**
     * Handle logical operators ($and, $or, $nor) in query filters.
     *
     * @param document  - Document to check.
     * @param operator  - Logical operator.
     * @param conditions - Array of conditions to evaluate.
     */
    private matchesLogicalOperator;
    /**
     * Handle comparison operators ($eq, $gt, $lt, etc.) in query filters.
     *
     * @param value     - Document field value.
     * @param operators - Comparison operators to apply.
     */
    private matchesComparisonOperators;
    /**
     * Sort documents by specified fields.
     *
     * @param documents - Documents to sort.
     * @param sort      - Sort specification (field: direction).
     */
    private sortDocuments;
    /**
     * Apply field projection to documents (include/exclude fields).
     *
     * @param documents  - Documents to project.
     * @param projection - Field projection specification.
     */
    private projectDocuments;
    /**
     * Use available indexes to optimize document filtering.
     *
     * @param documents - Documents to filter.
     * @param filter    - Query filter to apply.
     */
    private useIndexes;
}

/**
 * Base error class for all database-related errors.
 */
declare class DatabaseError extends Error {
    readonly code: string;
    readonly details?: any;
    /**
     * Create a new database error.
     *
     * @param message - Error message.
     * @param code    - Error code for programmatic handling.
     */
    constructor(message: string, code: string, details?: any);
}
/**
 * Error thrown when collection operations fail.
 */
declare class CollectionError extends DatabaseError {
    constructor(message: string, details?: any);
}
/**
 * Error thrown when document operations fail.
 */
declare class DocumentError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class SchemaError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class EncryptionError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class StorageError extends DatabaseError {
    constructor(message: string, details?: any);
}

/**
 * Generate a unique document ID using crypto.randomUUID().
 *
 * @returns A unique string identifier.
 */
declare function generateId(): string;
/**
 * Get the current timestamp as a Date object.
 *
 * @returns Current date and time.
 */
declare function generateTimestamp(): Date;
/**
 * Create metadata for a new document.
 *
 * @returns Object with ID and timestamps.
 */
declare function createDocumentMetadata(id?: string): {
    id: string;
    createdAt: Date;
    updatedAt: Date;
};

/**
 * Validate document data against a schema definition.
 *
 * @param data   - Document data to validate.
 * @param schema - Schema definition to validate against.
 * @throws Error if validation fails.
 */
declare function validateSchema(data: any, schema: Schema): void;
/**
 * Apply default values from schema to document data.
 *
 * @param data   - Document data.
 * @param schema - Schema with default values.
 * @returns Data with defaults applied.
 */
declare function applyDefaults(data: any, schema: Schema): any;
/**
 * Remove undefined values and sanitize document data.
 *
 * @param data - Raw document data.
 * @returns Sanitized document data.
 */
declare function sanitizeDocument(data: any): Omit<Document, '_id' | '_createdAt' | '_updatedAt'>;

/**
 * Convenience function for creating database instances.
 *
 * @param options - Database configuration options.
 * @returns Promise resolving to a database instance.
 */
declare function createDatabase(options: DatabaseOptions): Promise<NuboDB>;
declare const getDatabase: typeof NuboDB.getInstance;

export { BaseCollection, Collection, CollectionError, type CollectionOptions, DatabaseError, type DatabaseEvents, type DatabaseStats, type DeleteResult, type Document, DocumentError, type DocumentMetadata, DocumentOperations, type DocumentWithMetadata, EncryptionError, EncryptionManager, FileStorage, type FindResult, type IndexDefinition, type InsertManyResult, type InsertResult, NuboDB, QueryBuilder, type QueryBuilder$1 as QueryBuilderType, type QueryCondition, type QueryFilter, QueryOperations, type QueryOperator, type QueryOptions, type Schema, SchemaError, type SchemaField, StorageError, type Transaction, type UpdateResult, applyDefaults, createDatabase, createDocumentMetadata, NuboDB as default, generateId, generateTimestamp, getDatabase, sanitizeDocument, validateSchema };
